package com.sagacloud.utils;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by wo on 5/2/18.
 */
public class VendersUtil {

    public static JSONObject successJsonMsg(String msg) {
        JSONObject jsonObj = new JSONObject();
        jsonObj.put("result", "success");
        jsonObj.put("resultMsg", msg);
        return jsonObj;
    }

    public static JSONObject successJsonMsg(String msg, Object pojo){
        JSONObject obj = new JSONObject();
        if(pojo != null){
            obj = (JSONObject) JSONObject.toJSON(pojo);
        }
        JSONObject jsonObj = new JSONObject();
        jsonObj.put("result", "success");
        jsonObj.put("resultMsg", msg);
        jsonObj.put("content",obj);
        return jsonObj;
    }

    public static JSONObject successJsonMsg(String msg, List list) {
        JSONObject jsonObj = new JSONObject();
        jsonObj.put("result", "success");
        jsonObj.put("count",list.size());
        jsonObj.put("content",new JSONArray(list));
        jsonObj.put("resultMsg", msg);
        return jsonObj;
    }
    public static String returnJSONArray(JSONArray arr, Long totalSize) {
        JSONObject ret = JSONObject.parseObject("{\"Result\":\"success\",\"ResultMsg\":\"\", \"Content\":0}");
        ret.put("ResultMsg", "");
        ret.put("Count", (long) arr.size());
        if(totalSize != null)
            ret.put("TotalCount", totalSize);
        ret.put("Content", arr);
        return JSON.toJSONString(ret, SerializerFeature.WriteMapNullValue);
    }
    public static JSONObject successJsonMsg(String msg, JSONArray arr){
        JSONObject jsonObj = new JSONObject();
        jsonObj.put("result", "success");
        jsonObj.put("count",arr.size());
        jsonObj.put("content",arr);
        jsonObj.put("resultMsg", msg);
        return jsonObj;
    }
    public static JSONObject errorJsonMsg(String msg) {
        JSONObject jsonObj = new JSONObject();
        jsonObj.put("result", "failure");
        jsonObj.put("resultMsg", msg);
        return jsonObj;
    }

    public static Set<String> createSet(String... elements) {
        Set<String> set = new HashSet<>();
        for (String e : elements) {
            set.add(e);
        }
        return set;
    }

    public static Set<Object> getInfoSet(JSONArray content, String infoCode) {
        Set<Object> set = new HashSet<>();
        if(content == null)
            return set;
        for(int i = 0; i < content.size(); ++i){
            JSONObject single = content.getJSONObject(i);
            if(single == null)
                continue;
            JSONObject infos = single.getJSONObject("infos");
            if(infos == null)
                continue;
            Object value = infos.get(infoCode);
            if(value == null)
                continue;
            if(!set.contains(value))
                set.add(value);
        }
        return set;
    }

    /**
     * 　　DNA分析 　　拼字检查 　　语音辨识 　　抄袭侦测
     *
     * @createTime 2012-1-12
     */
    public static double levenshtein(String str1,String str2) {
        //计算两个字符串的长度。
        int len1 = str1.length();
        int len2 = str2.length();
        //建立上面说的数组，比字符长度大一个空间
        int[][] dif = new int[len1 + 1][len2 + 1];
        //赋初值，步骤B。
        for (int a = 0; a <= len1; a++) {
            dif[a][0] = a;
        }
        for (int a = 0; a <= len2; a++) {
            dif[0][a] = a;
        }
        //计算两个字符是否一样，计算左上的值
        int temp;
        for (int i = 1; i <= len1; i++) {
            for (int j = 1; j <= len2; j++) {
                if (str1.charAt(i - 1) == str2.charAt(j - 1)) {
                    temp = 0;
                } else {
                    temp = 1;
                }
                //取三个值中最小的
                dif[i][j] = min(dif[i - 1][j - 1] + temp, dif[i][j - 1] + 1,
                        dif[i - 1][j] + 1);
            }
        }
        //取数组右下角的值，同样不同位置代表不同字符串的比较
        //System.out.println("差异步骤："+dif[len1][len2]);
        //计算相似度
        double similarity =1 - (double) dif[len1][len2] / Math.max(str1.length(), str2.length());
        System.out.println("字符串\""+str1+"\"与\""+str2+"\"的比较, 相似度："+similarity);
        return similarity;
    }
    private static int min(int... is) {
        int min = Integer.MAX_VALUE;
        for (int i : is) {
            if (min > i) {
                min = i;
            }
        }
        return min;
    }

    public static ArrayList<String> createArrayList(String... elements) {
        ArrayList<String> list = new ArrayList<String>();
        for (String element : elements) {
            list.add(element);
        }
        return list;
    }
}
